const moment = require('moment');
const { simpleDate } = require("./Util");
const Calendar = require('./Helper/Calendar');
const ServiceConfig = require('./Helper/ServiceConfig');
const FlexRules = require('./Helper/FlexRules');

this.freqDayUsed = [];
this.randomDay = (calLenght) => {
    let random = Math.floor(Math.random() * calLenght);
    if (this.freqDayUsed.indexOf(random) < 0 && random != undefined) {
        this.freqDayUsed.push(random);
        return random;
    } else {
        return this.randomDay(calLenght, this.freqDayUsed);
    }
};

this.testRawData = function (rawData) {
    for (let i = 0; i < rawData.length; i++) {
        const curData = rawData[i];

        if (curData.SUCESSO != true) {
            throw 'Failed data, missing sql';
        }

    }
};

this.dow = {
    '1': 'IND_SEGUNDA',
    '2': 'IND_TERCA',
    '3': 'IND_QUARTA',
    '4': 'IND_QUINTA',
    '5': 'IND_SEXTA',
    '6': 'IND_SABADO',
    '7': 'IND_DOMINGO'
};

this.manageSeqSrvConf = ( srvConf ) => {

    let uniqueSeqSrvConf = [];
    let managedSeqSrvConf = [];

    srvConf.forEach(srCon => {
        let curSeq = srCon.NUM_SEQUENCIA;
        if (uniqueSeqSrvConf.indexOf(curSeq) < 0) {
            ServiceConfig.add(curSeq, {
                isFixColumn: srCon.IDOTMCOLUNAFIXA === 'S',
                ignoreColor: srCon.IDOTMNAOCONSCOR === 'S',
                ignorePrepMode: srCon.IDOTMNAOCONSMOD === 'S',
            })
            uniqueSeqSrvConf.push(curSeq);
            let sameSeqSrvConf = srvConf.filter(sc => sc.NUM_SEQUENCIA == curSeq);
            let filtredSeqSrvConf = {
                'ATRATIVIDADE': srCon.ATRATIVIDADE,
                'NUM_SEQUENCIA': srCon.NUM_SEQUENCIA,
                'COD_TIP_PRATO': srCon.COD_TIP_PRATO,
                'NOME_GRP_TIP_PRATO': srCon.NOME_GRP_TIP_PRATO,
                'NMTIPOPRAT': srCon.NMTIPOPRAT,
                'IND_OPCAO': srCon.IND_OPCAO,
                'COD_GRP_TIP_PRATO': srCon.COD_GRP_TIP_PRATO,
                'NUM_GRP_PRATO': []
            };

            sameSeqSrvConf.forEach(sssc => {
                filtredSeqSrvConf.NUM_GRP_PRATO.push(sssc.NUM_GRP_PRATO);
            });

            managedSeqSrvConf.push(filtredSeqSrvConf);
        }
    });

    return managedSeqSrvConf;
};

this.addPlatesAtSrvConf = ( srvConf, grupoPrato ) => {

    srvConf.forEach(currentConfig => {
        currentConfig.pratosDentroDosGrupos = [];
        currentConfig.NUM_GRP_PRATO.forEach(GRPPRATO => {
            let filtredPlatesOfCurGrPrat = grupoPrato.filter(gp => gp.NUM_GRP_PRATO == GRPPRATO);
            currentConfig.pratosDentroDosGrupos = currentConfig.pratosDentroDosGrupos.concat(filtredPlatesOfCurGrPrat);
        });
    });

    return srvConf;
};

let checkUtilizacao = (ut, day) => {
    let diaSitu = [day.IDPRIMSEM, day.IDSEMANA2, day.IDSEMANA3, day.IDSEMANA4, day.IDULTMSEM];
    let utilizacoes = [ut.IDPRIMSEM, ut.IDSEMANA2, ut.IDSEMANA3, ut.IDSEMANA4, ut.IDULTMSEM];
    let obrigatorio = false;
    let restrito = false;

    let okDSem = ut[day.diasemana] != 'R';
    let okFer = (day.IDFERIADO == 'S' && ut.IDFERIADO != 'R') || day.IDFERIADO == 'N';

    if (day.IDULTMSEM == "S" && diaSitu.filter(d => d == 'S').length == 2) {
        let indexes = [];
        diaSitu.forEach((d, k) => {
            if (d == 'S') {
                indexes.push(k);
            }
        });

        let newUt = [utilizacoes[indexes[0]], utilizacoes[indexes[1]]];
        obrigatorio = newUt[1] == 'O' ? true : ( newUt[1] == 'R' ? false : newUt[0] == 'O' );
        restrito = newUt[1] == 'R' ? true : ( newUt[1] == 'O' ? false : newUt[0] == 'R' );

    } else {
        obrigatorio = diaSitu.find((d, key) => d == 'S' && utilizacoes[key] == 'O');
        restrito = diaSitu.find((d, key) => d == 'S' && utilizacoes[key] == 'R');
    }
    return okDSem && (obrigatorio || !restrito) && okFer;
};

let setUtOnPrato = (ut, day) => {
    if (ut) {
        let diaSitu = [day.IDPRIMSEM, day.IDSEMANA2, day.IDSEMANA3, day.IDSEMANA4, day.IDULTMSEM];
        let utilizacoes = [ut.IDPRIMSEM, ut.IDSEMANA2, ut.IDSEMANA3, ut.IDSEMANA4, ut.IDULTMSEM];

        let obgDia = ut[day.diasemana] == 'O';
        let obgFer = ut.IDFERIADO == 'O' && day.IDFERIADO == 'S';

        let obgSem = utilizacoes[diaSitu.indexOf('S')] == 'O';

        if ((obgDia && !utilizacoes.includes('O') || (obgDia && obgSem))|| /* obgSem || */ obgFer) {
            return 'O';
        }
    }
    return 'P';
}

// Função para fixar dias na geração e facilitar analise de erros;
this.getFixedRandomDay = (calendario) =>{
    let curDate = [
        "19/01/2024","01/01/2024","09/01/2024","07/01/2024","13/01/2024","05/01/2024","15/01/2024","16/01/2024","10/01/2024","24/01/2024","25/01/2024","17/01/2024","20/01/2024","12/01/2024","28/01/2024","03/01/2024","22/01/2024","27/01/2024","04/01/2024","06/01/2024","11/01/2024","18/01/2024","31/01/2024","29/01/2024","26/01/2024","23/01/2024","30/01/2024","14/01/2024","02/01/2024","08/01/2024","21/01/2024"
    ][this.countFixedRandomDay];
    this.countFixedRandomDay++;
    return calendario.findIndex(({DTDATA}) => DTDATA.split(' ')[0] == curDate);
}

this.fillCardWithPlates = (receitaPrato, cardapio, configWithPlates, calendario, utilizacaoPratoSemana, prcConsumoPrato, managedServConfig) => {

    freqDayUsed = [];
    this.freqDayUsed = [];

    cardapio.datas = [];
    cardapio.seqInfo = managedServConfig;

    let daysGone = 0;
    this.countFixedRandomDay = 0;
    while (daysGone < calendario.length) {
        daysGone++;

        let randomDay = this.randomDay(calendario.length);
        let dayConverted = moment(calendario[randomDay].DTDATA, 'DD/MM/YYYY HH:mm:ss');

        let newDay = {};
        newDay.comensais = calendario[randomDay].QTDE_COMENSAIS;
        newDay.comenDIV = calendario[randomDay].COMENSAIS_DIVISAO;
        newDay.comenCLI = calendario[randomDay].QTDE_COMENSAIS_CLI;
        newDay.comenFUN = calendario[randomDay].QTDE_COMENSAIS_FUN;
        newDay.comenDOA = calendario[randomDay].QTDE_COMENSAIS_DOA;

        newDay.PER_PREPREPARO = calendario[randomDay].PER_PREPREPARO;

        newDay.IDSEMANA2 = calendario[randomDay].IDSEMANA2;
        newDay.IDSEMANA3 = calendario[randomDay].IDSEMANA3;
        newDay.IDSEMANA4 = calendario[randomDay].IDSEMANA4;

        newDay.IDPRIMSEM = calendario[randomDay].IDPRIMSEM;
        newDay.IDULTMSEM = calendario[randomDay].IDULTMSEM;
        newDay.IDFERIADO = calendario[randomDay].IDFERIADO;

        newDay.dia = dayConverted.format('DD/MM/YYYY');
        newDay.diasemana = this.dow[dayConverted.isoWeekday()];
        newDay.nDiasemana = dayConverted.isoWeekday();
        newDay.seq = {};

        configWithPlates.forEach(cwp => {
            newDay.seq[cwp.NUM_SEQUENCIA] = [];
        });

        for (const sequencial in newDay.seq) {
            let possuiGrupoRestrito = false;
            let pratosIrregular = false;
            if (newDay.seq.hasOwnProperty(sequencial)) {
                const pratos = newDay.seq[sequencial];
                let grupoSequencial = configWithPlates.filter(cwp => sequencial == cwp.NUM_SEQUENCIA);

                grupoSequencial.forEach(gs => {
                    let NUM_GRP_PRATO = gs.NUM_GRP_PRATO;

                    NUM_GRP_PRATO.forEach(gp => {
                        let utilizacaoPratoSemanaGRUPO = utilizacaoPratoSemana.find(ups => ups.NUM_GRP_PRATO == gp);

                        if (!utilizacaoPratoSemanaGRUPO || checkUtilizacao(utilizacaoPratoSemanaGRUPO, newDay)) {
                            let pratosGrupo = gs.pratosDentroDosGrupos.filter(pdg => pdg.NUM_GRP_PRATO == gp);

                            pratosGrupo.forEach(pg => {
                                let COD_PRATO = pg.COD_PRATO;
                                let recPrato = receitaPrato.find(rc => rc.COD_PRATO == COD_PRATO);
                                if (recPrato && recPrato.COD_PRATO) {
                                    //let consPrato = prcConsumoPrato.find(pcp => pcp.COD_PRATO == COD_PRATO && pcp.NRDIASEMANA == newDay.nDiasemana && pcp.COD_TIP_PRATO == gs.COD_TIP_PRATO);
                                    let consPrato = prcConsumoPrato.find(pcp => pcp.COD_PRATO == COD_PRATO);
                                    if (consPrato) {
                                        consPrato = consPrato.VRPECONSCARD;
                                    } else {
                                        consPrato = (100 / configWithPlates.filter(cwp => cwp.COD_TIP_PRATO == gs.COD_TIP_PRATO).length);
                                    }

                                    pratos.push({
                                        'percentualConsumo': consPrato,
                                        'utilizacao': setUtOnPrato(utilizacaoPratoSemanaGRUPO, newDay),
                                        'prato': COD_PRATO,
                                        'gpprato': gp,
                                        'contGpprato': pratosGrupo.length,
                                        'COMPLEX_PRATO': recPrato.COMPLEX_PRATO,
                                        'VLR_CUSTO_PRATO': recPrato.VLR_CUSTO_PRATO,
                                        'COR_PRATO': recPrato.COR_PRATO,
                                        'sequencial': sequencial,
                                        'IND_OPCAO': gs.IND_OPCAO,
                                        'COD_GRP_TIP_PRATO': gs.COD_GRP_TIP_PRATO,
                                        'NOME_GRP_TIP_PRATO': gs.NOME_GRP_TIP_PRATO,
                                        'COD_TIP_PRATO': gs.COD_TIP_PRATO
                                    });
                                } else {
                                    pratosIrregular = true;
                                }
                            });
                        } else if (utilizacaoPratoSemanaGRUPO || !checkUtilizacao(utilizacaoPratoSemanaGRUPO, newDay)){
                            possuiGrupoRestrito = true;
                        }
                    });
                    if(pratos.length == 0 && !pratosIrregular && possuiGrupoRestrito) {
                        newDay.buracoObrigatorio = newDay.buracoObrigatorio || new Set();
                        newDay.buracoObrigatorio.add(sequencial.toString());
                    }
                });

            }
        }
        cardapio.datas.push(newDay);
    }
    return cardapio;
};

this.feedReceitaWithGroups = (receitaPrato, grupoPrato) => {
    receitaPrato.forEach(rp => {
        let gpp = grupoPrato.find(gtp => gtp.COD_PRATO == rp.COD_PRATO);
        if (gpp) {
            rp.NUM_GRP_PRATO = gpp.NUM_GRP_PRATO;
        }
    });
    return receitaPrato;
};
/**
 * Retorna apenas pratos que possuem caracteristica cadastrada (ou NA)
 */
this.filterExcludedCarac = (cardapio, tpCorte, mdPreparo, ingPrinc) => {
    let tpc = tpCorte.map(i => {
        return i.NMAPRERECE;
    });
    let mdp = mdPreparo.map(i => {
        return i.NMFORMAPREPARO;
    });
    let inp = ingPrinc.map(i => {
        return i.NMINGREDIENTEPRINC;
    });

    cardapio.datas.forEach(dia => {
        let sequenciais = Object.keys(dia.seq);
        for (let i = 0; i < sequenciais.length; i++) {
            const seq = sequenciais[i];
            dia.seq[seq] = dia.seq[seq].filter(prato => {
                if ((mdp.includes(prato.NM_MODO_PREPARO)) && (tpc.includes(prato.NM_TP_CORTE))) {
                    if ((inp.includes(prato.NM_INGR_PRINCIPAL_1)) && (inp.includes(prato.NM_INGR_PRINCIPAL_2))) {
                        return prato;
                    }
                }

            });
        }
    });

    return cardapio;
};

this.filterCandidate = ( cardapiodeCandidatos, newReceitaPrato, res_grupo_prato ) => {
    cardapiodeCandidatos.datas.forEach(dia => {
        for (const sequencial in dia.seq) {
            if (dia.seq.hasOwnProperty(sequencial)) {
                let filteredPlate = [];
                dia.seq[sequencial].forEach(prato => {

                    let produtosPrato = newReceitaPrato.filter(nrp => prato.prato == nrp.COD_PRATO && prato.gpprato == nrp.NUM_GRP_PRATO);
                    let hasRestricao = false;
                    produtosPrato.forEach(pp => {
                        let cond = res_grupo_prato.find(rgp => pp.NUM_GRP_PRATO == rgp.NUM_GRP_PRATO && rgp.COD_PRODUTO == pp.COD_PRODUTO);
                        if (cond) {
                            hasRestricao = true;
                        }
                    });
                    if (!hasRestricao) {
                        prato.PRE_PREPARO = produtosPrato[0].PRE_PREPARO;
                        filteredPlate.push(prato);
                    }
                });

                dia.seq[sequencial] = filteredPlate.filter(fp => (dia.PER_PREPREPARO == 'N' && fp.PRE_PREPARO != 'S') || (dia.PER_PREPREPARO == 'S'));
            }
        }
    });
    return cardapiodeCandidatos;
};

this.logBuracos = (pratos, frequencias, seqInfo)=>{
    let seqFaltantes = seqInfo.filter(s=> pratos.map(p=> p.sequencial).indexOf(s.NUM_SEQUENCIA) < 0);
    let curFreq = frequencias.filter(f=> seqFaltantes.filter(s=> s.NUM_GRP_PRATO.indexOf(f.NUM_GRP_PRATO) > -1 ).length > 0);
    if(seqFaltantes.length) {
        return {
            'seqFaltantes': seqFaltantes.map(s=> {return {NUM_GRP_PRATO: s.NUM_GRP_PRATO, sequencial: s.NUM_SEQUENCIA, nometipoprato: s.NMTIPOPRAT}}),
            'frequencias': JSON.parse(JSON.stringify(curFreq)),
        };

    }
};

this.ajustSeq = (arr, frequencias, grupoPrato) => {
    let gpcombSom = {};
    let lastDESCRICAO = null;
    let dirGP = [];
    arr.forEach((c) => {
        if (!lastDESCRICAO || lastDESCRICAO != c.DESCRICAO) {
            lastDESCRICAO = c.DESCRICAO;
            dirGP = [];
        }

        let seqs = c.SEQ.split(';').filter(s => s.length > 0);
        let wseqs = [];
        seqs.forEach(sq => {
            if (wseqs.indexOf(sq) == -1) {
                wseqs.push(sq);
            }
        });
        c.SEQ = wseqs;

        let pivo = c.NRGRUPOPRATOSERV || c.COD_PRATO;
        const isCombWithDish = !c.NRGRUPOPRATOSERV_COMB;
        let dish = isCombWithDish ? grupoPrato.find(gp => gp.COD_PRATO == c.COD_PRATO_COMB) || {} : {};
        let { NUM_GRP_PRATO } = isCombWithDish ? dish : {NUM_GRP_PRATO: null};
        let gpcomb = c.NRGRUPOPRATOSERV_COMB || NUM_GRP_PRATO;
        if(isCombWithDish && !dish.COD_PRATO) {
            c.toRemove = true;
            return;
        }
        if (!dirGP.includes(gpcomb)) {
            let { MIN_MES_GRP } = frequencias.find(f => f.NUM_GRP_PRATO == gpcomb) || {MIN_MES_GRP: 99};
            c.mingpFreqComb = MIN_MES_GRP;
            gpcombSom[pivo] = (gpcombSom[pivo] || 0) + MIN_MES_GRP;
            dirGP.push(gpcomb);
        }
    });

    arr = arr.filter(comb => !comb.toRemove);
    arr.forEach(c => {
        let pivo = c.NRGRUPOPRATOSERV || c.COD_PRATO;
        c.mingpFreqComb = gpcombSom[pivo];
    });

    return arr;
};

var getDaysArray = (dateP) => {
    let month = dateP.split('/')[1];
    let year = dateP.split('/')[2];

    var monthIndex = month - 1;
    var date = new Date(year, monthIndex, 1);
    var result = [];
    while (date.getMonth() == monthIndex) {
        let day = `${((date.getDate() < 10) && ("0"+date.getDate())) || ""+date.getDate()}/${(((date.getMonth()+1) < 10) && ("0"+(date.getMonth() + 1))) || ""+(date.getMonth() + 1)}/${date.getFullYear()}`;
        result.push(day);
        date.setDate(date.getDate() + 1);
    }
    return result;
};

var getGrupoPratoNome = (grupoPrato) => {
    let grupoPratoNome = [];
    grupoPrato.forEach(gp => {
        if (grupoPratoNome.length == 0) {
            grupoPratoNome.push({
                'NOM_GRUPOPRATO': gp.NOM_GRUPOPRATO,
                'NUM_GRP_PRATO': gp.NUM_GRP_PRATO,
                'NMAGRUPGRPRAT': gp.NMAGRUPGRPRAT,
                'CDAGRUPGRPRAT': gp.CDAGRUPGRPRAT,
            });
        } else if (!grupoPratoNome.find(gpn => gpn.NUM_GRP_PRATO == gp.NUM_GRP_PRATO)) {
            grupoPratoNome.push({
                'NOM_GRUPOPRATO': gp.NOM_GRUPOPRATO,
                'NUM_GRP_PRATO': gp.NUM_GRP_PRATO,
                'NMAGRUPGRPRAT': gp.NMAGRUPGRPRAT,
                'CDAGRUPGRPRAT': gp.CDAGRUPGRPRAT,
            });
        }
    });
    return grupoPratoNome;
};

this.setProdsOnPratos = (cardapio, receitaPrato) => {
    for (let i = 0; i < cardapio.datas.length; i++) {
        let seq = Object.keys(cardapio.datas[i].seq);

        for (let j = 0; j < seq.length; j++) {

            for (let k = 0; k < cardapio.datas[i].seq[seq[j]].length; k++) {
                let prods = receitaPrato.filter(rp => rp.COD_PRATO == cardapio.datas[i].seq[seq[j]][k].prato);
                cardapio.datas[i].seq[seq[j]][k].prods = [];

                cardapio.datas[i].seq[seq[j]][k].INGR_PRINCIPAL_1 = prods[0].INGR_PRINCIPAL_1;
                cardapio.datas[i].seq[seq[j]][k].NM_INGR_PRINCIPAL_1 = prods[0].NM_INGR_PRINCIPAL_1;
                cardapio.datas[i].seq[seq[j]][k].INGR_PRINCIPAL_2 = prods[0].INGR_PRINCIPAL_2;
                cardapio.datas[i].seq[seq[j]][k].NM_INGR_PRINCIPAL_2 = prods[0].NM_INGR_PRINCIPAL_2;
                cardapio.datas[i].seq[seq[j]][k].MODO_PREPARO = prods[0].MODO_PREPARO;
                cardapio.datas[i].seq[seq[j]][k].NM_MODO_PREPARO = prods[0].NM_MODO_PREPARO;
                cardapio.datas[i].seq[seq[j]][k].TP_CORTE = prods[0].TP_CORTE;
                cardapio.datas[i].seq[seq[j]][k].NM_TP_CORTE = prods[0].NM_TP_CORTE;
                cardapio.datas[i].seq[seq[j]][k].ATRATIVIDADE = prods[0].ATRATIVIDADE;
                cardapio.datas[i].seq[seq[j]][k].NMATRATIVIDADE = prods[0].NMATRATIVIDADE;
                cardapio.datas[i].seq[seq[j]][k].VRPESOATRATIVIDADE = prods[0].VRPESOATRATIVIDADE;

                if (prods[0].NM_INGR_PRINCIPAL_1 == 'NA') {
                    cardapio.datas[i].seq[seq[j]][k].INGR_PRINCIPAL_1 = null;
                }

                if (prods[0].NM_INGR_PRINCIPAL_2 == 'NA') {
                    cardapio.datas[i].seq[seq[j]][k].INGR_PRINCIPAL_2 = null;
                }

                if (prods[0].NM_MODO_PREPARO == 'NA') {
                    cardapio.datas[i].seq[seq[j]][k].MODO_PREPARO = null;
                }

                if (prods[0].NM_TP_CORTE == 'NA') {
                    cardapio.datas[i].seq[seq[j]][k].TP_CORTE = null;
                }

                for (let l = 0; l < prods.length; l++) {
                    const prod = prods[l];

                    cardapio.datas[i].seq[seq[j]][k].prods.push({
                        'COD_PRODUTO': prod.COD_PRODUTO,
                        'NOM_PRODUTO': prod.NOM_PRODUTO,
                        'COD_ARV_PRODUTO': prod.COD_ARV_PRODUTO,
                        'GRAMAGEM': prod.GRAMAGEM
                    });
                }
            }

        }

    }
    return cardapio
};

this.addPlateData = (prato, grupoPratoNome, receitaPrato) => {
    let curgrupoPratoNome = grupoPratoNome.find(gpn => gpn.NUM_GRP_PRATO == prato.gpprato);
    let nomePrato = receitaPrato.find(rp => rp.COD_PRATO == prato.prato);
    if (curgrupoPratoNome) {
        prato.gppratoNome = curgrupoPratoNome.NOM_GRUPOPRATO;
        prato.NMAGRUPGRPRAT = curgrupoPratoNome.NMAGRUPGRPRAT;
        prato.CDAGRUPGRPRAT = curgrupoPratoNome.CDAGRUPGRPRAT;
    }
    if (nomePrato) {
        prato.nomePrato = nomePrato.NOM_PRATO;
    }

    return prato
};

this.getOtherOptionsForPlate = (prato, curSeq, ptrsOriginal, grupoPratoNome, receitaPrato) => {

    if (curSeq && ptrsOriginal && ptrsOriginal[curSeq]) {
        ptrsOriginal[curSeq].forEach(p => {
            p = this.addPlateData(p, grupoPratoNome, receitaPrato);
        });

        return prato.prato ? ptrsOriginal[curSeq].filter(po => po.prato != prato.prato) : ptrsOriginal[curSeq];
    }
};

this.removeUnusedDatafromCard = (cardapio) => {
    cardapio.datas.forEach(d => {
        delete d.ptrsOriginal;
        delete d.pratosBKP;
    });
    return cardapio;
};

Date.prototype.getWeekNumber = function () {
    var d = new Date(Date.UTC(this.getFullYear(), this.getMonth(), this.getDate()));
    var dayNum = d.getUTCDay() || 7;
    d.setUTCDate(d.getUTCDate() + 4 - dayNum);
    var yearStart = new Date(Date.UTC(d.getUTCFullYear(), 0, 1));
    return Math.ceil((((d - yearStart) / 86400000) + 1) / 7)
};

this.setDatePropsOnCalendario = (dates) => {
    let daysOnWeeks = {};

    dates.forEach(d => {
        const dateInstance = moment(d.DTDATA, 'DD/MM/YYYY');
        Calendar.addDay(d.DTDATA, dateInstance.toDate().getDate());
        let wd = dateInstance.day();
        if (!daysOnWeeks[wd]) {
            daysOnWeeks[wd] = [];
        }
        daysOnWeeks[wd].push(d);
    });

    daysOnWeeks = Object.entries(daysOnWeeks);
    dates.forEach(d => {
        let date = moment(d.DTDATA.substring(0, 10), 'DD/MM/YYYY');
        let idDiaSemana = daysOnWeeks.find(dow => dow[0] == date.weekday())[1];
        let indexDiaSemana = idDiaSemana.indexOf(d);

        d.IDPRIMSEM = indexDiaSemana == 0 && indexDiaSemana != idDiaSemana.length - 1 ? 'S' : 'N';
        d.IDSEMANA2 = indexDiaSemana == 1 && indexDiaSemana != idDiaSemana.length - 1 ? 'S' : 'N';
        d.IDSEMANA3 = indexDiaSemana == 2 && indexDiaSemana != idDiaSemana.length - 1 ? 'S' : 'N';
        d.IDSEMANA4 = indexDiaSemana == 3 ? 'S' : 'N';
        d.IDULTMSEM = indexDiaSemana == idDiaSemana.length - 1 ? 'S' : 'N';
    });
    return dates;
};

this.genCard = (cardapio, cardapioToAval, seqOrder, grupoPrato, receitaPrato, meta, utilizacaoPratoSemana, conjuntoPorcao) => {
    let comTotal = 0;
    let comDIVTotal = 0;
    let puAculm = 0;

    let colFixBefore = [];
    let tabela = [];
    let colFixAfter = [];
    let mes = getDaysArray(cardapioToAval.mes);
    let grupoPratoNome = getGrupoPratoNome(grupoPrato);

    Object.entries(cardapio.log ? cardapio.log.precardData : {}).forEach(d => {
        d[1].forEach((p, key) => {
            d[1][key] = this.addPlateData(p, grupoPratoNome, receitaPrato);
            d[1][key] = (p.sequencial + ' - ' + p.gppratoNome + ' - ' + p.nomePrato);
        });
    });

    Object.entries(cardapio.log ? cardapio.log.antesPoscard : {}).forEach(d => {
        d[1].forEach((p, key) => {
            d[1][key] = this.addPlateData(p, grupoPratoNome, receitaPrato);
            d[1][key] = (p.sequencial + ' - ' + p.gppratoNome + ' - ' + p.nomePrato);
        });
    });

    Object.entries(cardapio.log ? cardapio.log.depoisPoscard : {}).forEach(d => {
        d[1].forEach((p, key) => {
            d[1][key] = this.addPlateData(p, grupoPratoNome, receitaPrato);
            d[1][key] = (p.sequencial + ' - ' + p.gppratoNome + ' - ' + p.nomePrato);
        });
    });

    colFixBefore[0] = ['Comensais Estimados', 'Data'];
    colFixAfter[0] = ['PU. DIA', 'PU. ACUMULADO'];

    let nmtpprato = [];
    seqOrder.forEach(so => {
        nmtpprato.push({
            'name': so.NMTIPOPRAT,
            'seq': so.NRSEQCNFCARD
        });
    });

    tabela[0] = nmtpprato;

    cardapioToAval.datas.sort((a, b) => {
        if (a.diaOrd > b.diaOrd) {
            return 1
        } else {
            return -1
        }
    });

    let pcSom = 0
    let puCont = 0;
    let puac = 0;
    let custoAcumulado = 0;
    mes.forEach(diaM => {
        let dia = cardapioToAval.datas.find(d => d.dia == diaM);
        let pratos = [];
        let comensais = dia ? dia.comensais : 0;
        let comenDIV = dia ? dia.comenDIV : 0;
        comTotal = parseFloat(comTotal) + parseFloat(comensais);
        comDIVTotal = parseFloat(comDIVTotal) + parseFloat(comenDIV);

        let data = dia ? dia.dia : diaM;

        let cusDia = dia ? parseFloat(dia.pratos.mediaCusto) * parseFloat(comensais) : 0;
        custoAcumulado += cusDia;
        puac = custoAcumulado / comDIVTotal;
        let pudia = parseFloat(comenDIV) === 0 ? 0 : cusDia / parseFloat(comenDIV);
        pcSom = parseFloat(pcSom) + parseFloat(pudia);

        colFixBefore.push([comensais, data]);

        nmtpprato.forEach(tpprato => {
            let curSeq = tpprato.seq;

            if (dia && dia.pratos.find(p => p.sequencial == curSeq)) {
                dia.pratos.forEach(p => {
                    let otherOptions = this.getOtherOptionsForPlate(p, p.sequencial, dia.ptrsOriginal, grupoPratoNome, receitaPrato);
                    p = this.addPlateData(p, grupoPratoNome, receitaPrato);
                    p.otherOptions = otherOptions && otherOptions.length > 0 ? [1] : [];
                    p.pratosBKP = this.getOtherOptionsForPlate(p, p.sequencial, dia.pratosBKP, grupoPratoNome, receitaPrato);
                    if(p.sequencial == curSeq && !p.isPartialCombination) {
                        p.buracoObrigatorio = false;
                        p.data = data;
                    }
                });
                pratos.push(dia.pratos.find(p => p.sequencial == curSeq));
            } else {
                let plateData = null;
                let possuiBuracoObrigatorio = dia && dia.buracoObrigatorio && dia.buracoObrigatorio.has(curSeq.toString()) ? true : false;
                if (dia && dia.ptrsOriginal) {
                    let otherOptions = this.getOtherOptionsForPlate({}, curSeq, dia.ptrsOriginal, grupoPratoNome, receitaPrato);
                    plateData = {
                        data,
                        'sequencial': curSeq,
                        'otherOptions': otherOptions && otherOptions.length > 0 ? [1] : [],
                        'buracoObrigatorio': possuiBuracoObrigatorio,
                        'pratosBKP': this.getOtherOptionsForPlate({}, curSeq, dia.pratosBKP, grupoPratoNome, receitaPrato)
                    }
                } else {
                    plateData = {
                         data,
                        'sequencial': curSeq,
                        'buracoObrigatorio': possuiBuracoObrigatorio
                    }
                }
                pratos.push(plateData);
            }
        });

        colFixAfter.push([pudia, puac]);
        if (dia) {
            puCont++;
        }

        tabela.push(pratos);
    });

    puAculm = colFixAfter[colFixAfter.length - 1][1];
    cardapio = this.removeUnusedDatafromCard(cardapio);
    return {
        rebuildCard: {
            conjuntoPorcao,
        },
        cardapio,
        'tabela': {
            'comensais_datas': colFixBefore,
            'pratos': tabela,
            'pudia_puacu': colFixAfter,
            'log': cardapio.log
        },
        'cust_total': comDIVTotal * puAculm,
        'comensal_total': comDIVTotal,
        'pu_aculm': puAculm,
        'custo_pdr': meta,
        'dif_custo': meta - puAculm,
        grupoPrato,
        utilizacaoPratoSemana
    };

};

this.setCardCong = function (cardCongData) {
    if (!cardCongData || cardCongData.length == 0) {
        return null;
    }
    let datasCong = [];
    let curData = null;
    let cardCong = [];
    let pratosUs = [];
    let prodUs = [];

    do {
        if (!curData) {
            curData = cardCongData[0];
        }
        let dia = cardCongData.filter(cardItem => cardItem.DTDATA == curData.DTDATA);

        let configDia = [];
        let curPrato = null;
        let pratosRun = [];


        do {
            if (!curPrato) {
                let nextPrato = dia.find(p => p.COD_PRATO != null);
                curPrato = nextPrato ? Object.assign({}, nextPrato) : null;
            }
            if (curPrato) {
                let prods = dia.filter(p => p.COD_PRATO == curPrato.COD_PRATO);
                curPrato.prods = [];
                prods.forEach(pd => {
                    let prod = {
                        'COD_PRODUTO': pd.COD_PRODUTO,
                        'NOM_PRODUTO': pd.NOM_PRODUTO,
                        'COD_ARV_PRODUTO': pd.COD_ARV_PRODUTO,
                        'GRAMAGEM': pd.GRAMAGEM
                    };
                    curPrato.prods.push(prod);
                    prodUs.push(prod);
                });
                curPrato.fromPreCard = true;
                curPrato.codprato = curPrato.COD_PRATO;
                curPrato.prato = curPrato.COD_PRATO;
                curPrato.gpprato = curPrato.NUM_GRP_PRATO;
                curPrato.nomePrato = curPrato.NOM_PRATO;
                curPrato.nrPratoCardapio = curPrato.NUM_SEQUENCIA;
                curPrato.otherOptions = [];
                curPrato.percentualConsumo = curPrato.VRPECONSCARD;
                curPrato.sequencial = curPrato.NUM_SEQUENCIA;
                curPrato.tipoPrato = curPrato.CDTIPOPRAT;
                curPrato.COD_TIP_PRATO = curPrato.CDTIPOPRAT;
                curPrato.gppratoNome = curPrato.NOME_GRP_PRATO;
                curPrato.unidadePrato = curPrato.CDFILAUXPRAT;
                curPrato.utilizacao = curPrato.UTILIZACAO;

                pratosUs.push(curPrato);
                configDia.push(curPrato);
                pratosRun.push(curPrato.NUM_SEQUENCIA + '' + curPrato.COD_PRATO);
                curPrato = dia.find(p => p.COD_PRATO != null && !pratosRun.includes(p.NUM_SEQUENCIA + '' + p.COD_PRATO));
            }

        } while (curPrato);

        let anyCom = dia.find(d => d.QTDE_COMENSAIS != null);
        cardCong.push({
            comenDIV: anyCom ? anyCom.COMENSAIS_DIVISAO : '0',
            comenCLI: anyCom ? anyCom.QTDE_COMENSAIS_CLI : '0',
            comenFUN: anyCom ? anyCom.QTDE_COMENSAIS_FUN : '0',
            comenDOA: anyCom ? anyCom.QTDE_COMENSAIS_DOA : '0',
            comensais: anyCom ? anyCom.QTDE_COMENSAIS : '0',
            dia: curData.DTDATA,
            diaOrd: moment(curData.DTDATA, 'DD/MM/YYYY').format('YYYYMMDD'),
            pratos: configDia
        });

        datasCong.push(simpleDate(curData.DTDATA));
        let nextDate = cardCongData.find(cardItem => !datasCong.includes(simpleDate(cardItem.DTDATA)));
        curData = nextDate ? Object.assign({}, nextDate) : null;
    } while (curData);

    let total = 0;
    cardCong.forEach(dia => {
        let mediaCusto = 0;
        for (let i = 0; i < dia.pratos.length; i++) {
            const prato = dia.pratos[i];
            mediaCusto = mediaCusto + (parseFloat(prato.VLR_CUSTO_PRATO) * (parseFloat(prato.percentualConsumo) / 100));
        }
        dia.pratos['mediaCusto'] = mediaCusto;
        total += mediaCusto;
    });

    return {
        pratosUs,
        prodUs,
        cardCong,
        datasCong,
        total
    };

};

this.getPratosFromServOrigem = function (sequencialCenarioOriginal, parsedCardapio) {
    let diaspratOrigem = [];
    (parsedCardapio && parsedCardapio.datas || []).forEach(d => {
        let pratos = [];

        d.pratos.filter(p => sequencialCenarioOriginal.find(sco => sco.NRSEQCNFCARDORIG == p.sequencial)).forEach(p => {
            pratos.push({
                prato: p.codprato,
                sequencial: sequencialCenarioOriginal.find(sco => sco.NRSEQCNFCARDORIG == p.sequencial).NRSEQCNFCARD
            });
        });

        diaspratOrigem.push({
            dia: d.dia,
            pratos
        });
    });

    return diaspratOrigem;
};

this.setPratosFromServOrigem = function (cardapio, diasCardOrigem) {
    let isObject = ( item ) => item && typeof item.hasOwnProperty == 'function';
    cardapio.datas.forEach((d, key) => {
        let dorigin = diasCardOrigem.find(dco => dco.dia == d.dia);
        (dorigin && dorigin.pratos || []).forEach(dop => {
            let data = cardapio.datas[key] || {};
            let pratos = data.seq || {};
            if(isObject(pratos) && Array.isArray(pratos[dop.sequencial])) {
                cardapio.datas[key].seq[dop.sequencial] = cardapio.datas[key].seq[dop.sequencial].filter(p => {
                    let isMatched = p.prato == dop.prato
                    if(isMatched) {
                        p.utilizacao = 'O';
                    }
                    return isMatched;
                });
            }
        });
    });
    return cardapio;
};