const analizeRestIng = ( restIng ) => {
    let restIngMap = new Map();
    restIng.forEach( rest => {
        let allTypeDish = rest.TIPOPRAT.substring(0, rest.TIPOPRAT.length-1).split(';');
        
        let ingredient = rest.NRINGREDIENTEPRINC || 'XX';
        const {IDMESMODIA, IDDIASEGUIDO, INGR_PRINCIPAL_1, INGR_PRINCIPAL_2} = rest;
        const DENY_REPEAT_ING_1 = INGR_PRINCIPAL_1 === 'S';
        const DENY_REPEAT_ING_2 = INGR_PRINCIPAL_2 === 'S';
        const DENY_SAME_DAY = IDMESMODIA === 'S';
        allTypeDish.forEach( cdtipoprat => {
            restIngMap.set(`${cdtipoprat}_${ingredient}`, {DENY_SAME_DAY, IDDIASEGUIDO, DENY_REPEAT_ING_1, DENY_REPEAT_ING_2});
        });
        
    });
    return restIngMap;
}

const isValidIngredient = ( dishes, rest1, rest2, ing1, ing2 ) => {
    const hasDishes = Array.isArray(dishes) && dishes.length;
    if( hasDishes && (rest1 || rest2) && (ing1 || ing2) ) {
        let hasInvalidDishes = dishes.some( dish => {
            return ( rest1 && rest1.DENY_REPEAT_ING_1 && dish.INGR_PRINCIPAL_1 == ing1 ) 
                || ( rest2 && rest2.DENY_REPEAT_ING_2 && dish.INGR_PRINCIPAL_2 == ing2 )
        });
        return !hasInvalidDishes;
    }
    return true;
};

this.isAllowedIngredient = ( menu, ing1, ing2, cdtipoprat, restIng ) => {
    if(!restIng || (Array.isArray(restIng) && restIng.length === 0) || (!ing1 && !ing2)) return true;

    const restIngMap = analizeRestIng( restIng );
    const { yesterday, today, tomorrow } = menu;
    let rest1 = restIngMap.get(`${cdtipoprat}_${ing1}`) || restIngMap.get(`${cdtipoprat}_XX`) || null;
    let rest2 = restIngMap.get(`${cdtipoprat}_${ing2}`) || restIngMap.get(`${cdtipoprat}_XX`) || null;
    const validateRest1SameDay = rest1 && rest1.DENY_SAME_DAY;
    const validateRest2SameDay = rest2 && rest2.DENY_SAME_DAY;
    if(!isValidIngredient( today, (validateRest1SameDay ? rest1 : null), (validateRest2SameDay ? rest2 : null), ing1, ing2)) return false;

    if(rest1 && rest1.IDDIASEGUIDO !== 'S') rest1 = null;
    if(rest2 && rest2.IDDIASEGUIDO !== 'S') rest2 = null;

    const isValidYesterday = isValidIngredient(yesterday, rest1, rest2, ing1, ing2);
    if(!isValidYesterday) return false;
    const isValidTomorrow = isValidIngredient(tomorrow, rest1, rest2, ing1, ing2);
    if(!isValidTomorrow) return false;

    return true;
}