const moment = require('moment');

/**
 * @param date | String
 * @param format | String
 */
this.getYesterday = ( date, format = 'DD/MM/YYYY' ) => {
    if(typeof date !== 'string') throw new TypeError('date is not String');
    if(typeof format !== 'string') throw new TypeError('format is not String');
    return moment(date, format).subtract(1, 'days').format(format);
}

/**
 * @param date | String
 * @param format | String
 */
this.getTomorrow = ( date, format = 'DD/MM/YYYY' ) => {
    if(typeof date !== 'string') throw new TypeError('date is not String');
    if(typeof format !== 'string') throw new TypeError('format is not String');
    return moment(date, format).add(1, 'days').format(format);
}

/**
 * @param date | String
 * @param format | String
 * 
 * @return Object
 */
this.getDayStartEndWeek = ( date, format = 'DD/MM/YYYY', onlySameMonth = true ) => {
    const dayObj = moment(date, format);
    let startWeek = dayObj.clone().startOf('week');
    let endWeek = dayObj.clone().endOf('week');
    if( onlySameMonth ) {
        if(parseInt(startWeek.format('YYYYMM')) < parseInt(dayObj.format('YYYYMM'))) {
            startWeek = dayObj.clone().startOf('month');
        }
        if(parseInt(endWeek.format('YYYYMM')) > parseInt(dayObj.format('YYYYMM'))) {
            endWeek = dayObj.clone().endOf('month');
        }
    }
    return {
        START_DAY: parseInt(startWeek.format('DD')),
        END_DAY: parseInt(endWeek.format('DD'))
    };
};

/**
 * @param date | String
 * @param format | String
 */
this.getDay = ( date, format = 'DD/MM/YYYY' ) => {
    return parseInt(moment(date, format).format('DD'));
};

/**
 * @param date | String
 * @param format | String
 */
this.getWeek = ( date, format = 'DD/MM/YYYY') => {
    return moment(date, format).week();
};

/**
 * @param day | Integer, String
 * @param referenceDate | String
 * @param format | String
 * 
 * @return String
 */
this.fillDateFromDay = (day, referenceDate, format = 'DD/MM/YYYY') => {
    const MONTH_YEAR = moment(referenceDate, format).format('MM/YYYY');
    const DAY = day.toString().padStart('0', 2);
    return moment(`${DAY}/${MONTH_YEAR}`, 'DD/MM/YYYY').format(format);
};